// Copyright (c) 2019 WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
//
// WSO2 Inc. licenses this file to you under the Apache License,
// Version 2.0 (the "License"); you may not use this file except
// in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
import ballerina/io;

function testNilArrayFill(int index, () value) returns ()[] {
    ()[] ar = [];
    ar[index] = value;
    return ar;
}

function testBooleanArrayFill(int index, boolean value) returns boolean[] {
    boolean[] ar = [];
    ar[index] = value;
    return ar;
}

function testIntArrayFill(int index, int value) returns int[] {
    int[] ar = [];
    ar[index] = value;
    return ar;
}

function testFloatArrayFill(int index, float value) returns float[] {
    float[] ar = [];
    ar[index] = value;
    return ar;
}

function testDecimalArrayFill(int index, decimal value) returns decimal[] {
    decimal[] ar = [];
    ar[index] = value;
    return ar;
}

function testStringArrayFill(int index, string value) returns string[] {
    string[] ar = [];
    ar[index] = value;
    return ar;
}

type Person record {
    string name;
    int age;
};

function testArrayOfArraysFill(int index) returns Person[][] {
    Person p = {name:"John", age:25};
    Person[] personArr = [p, p];
    Person[][] ar = [];
    ar[index] = personArr;
    return ar;
}

function testTupleArrayFill(int index) returns (string, int)[] {
    (string, int) tup = ("Hello World!", 100);
    (string, int)[] ar = [];
    ar[index] = tup;
    return ar;
}

function testMapArrayFill(int index, map<any> value) returns map<any>[] {
    map<any>[] ar = [];
    ar[index] = value;
    return ar;
}

type Foo record {
    string s = "foo";
    int i;
    float f?;
    boolean b;
    decimal d;
    () n;
    (string, int) t;
    map<any> m;
    xml x;
    byte bt;
    json j;
    anydata ad;
};

function testRecordArrayFill(int index) returns Foo[] {
    map<any> m = {"1": 1};
    Foo value = {
        i: 10,
        f: 12.34,
        b: true,
        d: 23.45,
        n: (),
        t: ("Pubudu", 27),
        m: m,
        x: xml ` `,
        bt: 5,
        j: {name: "Pubudu"},
        ad: 10
    };
    Foo[] ar = [];
    ar[index] = value;
    return ar;
}

type Employee record {
    int id;
    string name;
    float salary;
};

function testTableArrayFill(int index) returns (table<Employee>[], string) {
    table<Employee> tbEmployee = table { { key id, name, salary }, [{1, "John", 50000}] };
    table<Employee>[] ar = [];
    ar[index] = tbEmployee;

    string name = "";
    foreach var tab in ar {
        foreach var row in tab {
            name += row.name;
        }
    }

    return (ar, name);
}

function testXMLArrayFill(int index) returns xml[] {
    xml value = xml `<name>Pubudu</name>`;
    xml[] ar = [];
    ar[index] = value;
    return ar;
}

function testTypedescArrayFill(int index) returns typedesc[] {
    typedesc[] ar = [];
    ar[index] = int;
    return ar;
}

function testUnionArrayFill1(int index) returns (string|int|Person|())[] {
    (string|int|Person|()) value = "Hello World!";
    (string|int|Person|())[] ar = [];
    ar[index] = value;
    return ar;
}

function testUnionArrayFill2(int index) returns (string|string)[] {
    (string|string) value = "Hello World!";
    (string|string)[] ar = [];
    ar[index] = value;
    return ar;
}

function testUnionArrayFill3(int index) returns (Person|Person|())[] {
    (Person|Person) value = {name:"John", age:25};
    (Person|Person|())[] ar = [];
    ar[index] = value;
    return ar;
}

// disabled due to https://github.com/ballerina-platform/ballerina-lang/issues/13612
//type LiteralsAndType 1|2|int;

//function testUnionArrayFill4(int index) returns LiteralsAndType[] {
//    LiteralsAndType value = 1;
//    LiteralsAndType[] ar = [];
//    ar[index] = value;
//    return ar;
//}

function testOptionalTypeArrayFill(int index) returns string?[] {
    string? value = "Hello World!";
    string?[] ar = [];
    ar[index] = value;
    return ar;
}

function testAnyArrayFill(int index) returns any[] {
    Person p = {name:"John", age:25};
    any value = p;
    any[] ar = [];
    ar[index] = value;
    return ar;
}

function testAnydataArrayFill(int index) returns anydata[] {
    Person p = {name:"John", age:25};
    anydata value = p;
    anydata[] ar = [];
    ar[index] = value;
    return ar;
}

function testByteArrayFill(int index, byte value) returns byte[] {
    byte[] ar = [];
    ar[index] = value;
    return ar;
}

function testJSONArrayFill(int index) returns json[] {
    json value = {name:"John", age:25};
    json[] ar = [];
    ar[index] = value;
    return ar;
}

type digit 0|1|2|3|4|5|6|7|8|9;

function testFiniteTypeArrayFill1(int index) returns digit[] {
    digit value = 5;
    digit[] ar = [];
    ar[index] = value;
    return ar;
}

type FP 0.0|1.2|2.3;

function testFiniteTypeArrayFill2(int index) returns FP[] {
    FP value = 1.2;
    FP[] ar = [];
    ar[index] = value;
    return ar;
}

type bool false|true;

function testFiniteTypeArrayFill3(int index) returns bool[] {
    bool[] ar = [];
    ar[index] = true;
    return ar;
}

type state false|true|();

function testFiniteTypeArrayFill4(int index) returns state[] {
    state[] ar = [];
    ar[index] = true;
    return ar;
}

// disabled due to https://github.com/ballerina-platform/ballerina-lang/issues/13612

//const decimal ZERO = 0.0;
//const decimal ONE_TWO = 1.2;
//const decimal TWO_THREE = 2.3;

//type DEC ZERO|ONE_TWO|TWO_THREE;

//function testFiniteTypeArrayFill5(int index) returns DEC[] {
//    DEC value = 1.2;
//    DEC[] ar = [];
//    ar[index] = value;
//    return ar;
//}

type One 1;

function testSingletonTypeArrayFill(int index) returns One[] {
    One[] ar = [];
    ar[index] = 1;
    return ar;
}
